import UIKit
import WebKit
import Evergage

let SCREEN_WIDTH = UIScreen.main.bounds.size.width
let SCREEN_HEIGHT = UIScreen.main.bounds.size.height
let BACKGROUND_COLOR = UIColor.label.withAlphaComponent(0.3)
let POPUP_VIEW_WIDTH = 0.85 * SCREEN_WIDTH
let POPUP_VIEW_COLOR = UIColor.systemBackground
let POPUP_VIEW_CORNER_RADIUS = CGFloat(10.0)
let LABEL_WIDTH = 0.9 * POPUP_VIEW_WIDTH
let LABEL_X = (POPUP_VIEW_WIDTH - LABEL_WIDTH) / 2
let INTERVAL = 0.022 * SCREEN_HEIGHT

class CustomPopupAction : NSObject {
    var handler: ((_ action: CustomPopupAction)->())?
    
    init( handler: ((_ action: CustomPopupAction) -> ())?){
        self.handler = handler
    }
}

enum CustomPopupControlerStyle {
    case popup
}

class CustomPopupController: UIViewController, UIViewControllerTransitioningDelegate, WKNavigationDelegate, WKUIDelegate, WKScriptMessageHandler {
    var popupTitle: String?
    var popupMessage: String?
    var popupImage: String?
    var webViewFile: String?
    var titleText: String?
    var titleColor: String?
    var titleFontSize: String?
    var subtitleText: String?
    var bannerUrl: String?
    var buttonUrl: String?
    var buttonText: String?
    var closeButtonText: String?
    var buttonColorTheme: String?

    internal convenience init(
        webViewFile: String?,
        titleText: String?,
        titleColor: String?,
        titleFontSize: String?,
        subtitleText: String?,
        bannerUrl: String?,
        buttonUrl: String?,
        buttonText: String?,
        closeButtonText: String?,
        buttonColorTheme: String?
    ) {
        self.init()
        self.webViewFile = webViewFile
        self.titleText = titleText
        self.titleColor = titleColor
        self.titleFontSize = titleFontSize
        self.subtitleText = subtitleText
        self.bannerUrl = bannerUrl
        self.buttonUrl = buttonUrl
        self.buttonText = buttonText
        self.closeButtonText = closeButtonText
        self.buttonColorTheme = buttonColorTheme
        
        let tapGestureRecognizer = UITapGestureRecognizer(target: self, action: #selector(self.dismissPopup(withGestureRecognizer:)))
        self.popupBackground.addGestureRecognizer(tapGestureRecognizer)
    }
    
    @objc func dismissPopup( withGestureRecognizer event: UITapGestureRecognizer ) {
        dismiss(animated: true, completion: {
            let action = self.actions[0]
            if let handler = action.handler{
                handler(action)
            }
        })
    }
    var customWebView = WKWebView()
    var popupBackground = UIView()
    var popupView = UIView()
    var actions = [CustomPopupAction]()
    
    var itemData: [Any] = []
    func addAction(_ action:CustomPopupAction){
        actions.append(action)
    }
    override func viewDidLoad() {
        popupView.autoresizesSubviews = false
        
        let webPagePreference = WKWebpagePreferences()
        let userController = WKUserContentController()
        let preferences = WKPreferences()
        if #available(iOS 14.0, *) {
            webPagePreference.allowsContentJavaScript = true
        } else {
            preferences.javaScriptEnabled = true
        }
        
        customWebView.frame = .zero
        customWebView.configuration.defaultWebpagePreferences = webPagePreference
        customWebView.configuration.preferences = preferences
        customWebView.configuration.userContentController = userController
        customWebView.configuration.userContentController.add(self, name: "callbackHandler")
        customWebView.uiDelegate = self
        customWebView.scrollView.bounces = false
        
        do{
            guard let path: String = Bundle.main.path(forResource: self.webViewFile, ofType: "html") else { return }
            let url = URL(fileURLWithPath: path)
            let request = URLRequest(url: url)
            customWebView.navigationDelegate = self
            customWebView.load(request)
        }
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        _ = initViewLayout
    }
    
    private lazy var initViewLayout : Void = {
        setUIParts()
    }()
    
    private func setUIParts() {
        var tmpY = 0.0
    
        setUpWebView(tmpY: tmpY)
        tmpY += customWebView.frame.size.height + INTERVAL
        setUpPopupView(tmpY: tmpY)
        popupBackground.backgroundColor = BACKGROUND_COLOR
        popupBackground.addSubview(popupView)
        
        self.view = popupBackground
    }

    private func setUpWebView(tmpY: CGFloat) {
        self.customWebView.frame = CGRect(x: LABEL_X, y: tmpY, width: LABEL_WIDTH, height: 300)
        self.customWebView.sizeToFit()
        self.customWebView.center = CGPoint(x: POPUP_VIEW_WIDTH/2, y: customWebView.center.y)
    }

    private func setUpPopupView(tmpY: CGFloat) {
        popupView.backgroundColor = POPUP_VIEW_COLOR
        popupView.layer.cornerRadius = POPUP_VIEW_CORNER_RADIUS
        popupView.frame.size = CGSize(width: POPUP_VIEW_WIDTH, height: tmpY)
        popupView.center = CGPoint(x: SCREEN_WIDTH/2, y: SCREEN_HEIGHT/2)
        // ポップアップビューにWebViewを追加
        popupView.addSubview(customWebView)
    }

    private func encodeJson(campData: PopupItem) -> String {
        let encoder = JSONEncoder()
        do{
            let data = try encoder.encode(campData)
            let jsonString:String = String(data: data, encoding: .utf8)!
            return jsonString
        } catch {
            return "error"
        }
    }
    
    func userContentController(_ userContentController: WKUserContentController, didReceive message: WKScriptMessage) {
        if(message.name == "callbackHandler") {
            dismiss(animated: true, completion: {
                let action = self.actions[0]
                if let handler = action.handler{
                    handler(action)
                }
            })
        }
    }
    
    func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {

        if navigationAction.navigationType == WKNavigationType.linkActivated {
            // 詳しくみるボタンのリンクURLを取得
            print(navigationAction.request.url!)

            decisionHandler(WKNavigationActionPolicy.cancel)
            return
        }
        print("no link")
        decisionHandler(WKNavigationActionPolicy.allow)
    }
     

    func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {

        let popupItem = PopupItem(
            titleText: self.titleText!,
            titleColor: self.titleColor ?? "#333333",
            titleFontSize: self.titleFontSize ?? "28px",
            subtitleText: self.subtitleText!,
            bannerUrl: self.bannerUrl!,
            buttonUrl: self.buttonUrl!,
            buttonText: self.buttonText!,
            closeButtonText: self.closeButtonText!,
            buttonColorTheme: self.buttonColorTheme!
        )
        let jsonObject = encodeJson(campData: popupItem)
        itemData.append( jsonObject )
    
        //　WebviewにMobile Data Campaignのデータを渡す
        customWebView.evaluateJavaScript("loadVariables(\(itemData))", completionHandler: nil)
        
      self.customWebView.evaluateJavaScript("document.documentElement.scrollHeight", completionHandler: { (height, error) in
            self.customWebView.frame.size.height = (height as? CGFloat)!
            self.setUpPopupView(tmpY: height as! CGFloat)
        })
    
    }
}

struct PopupItem: Codable {
    let titleText: String
    let titleColor: String
    let titleFontSize: String
    let subtitleText: String
    let bannerUrl: String
    let buttonUrl: String
    let buttonText: String
    let closeButtonText: String
    let buttonColorTheme: String
}
